/*
        LDR ToneMapper

        Copyright (C) 2009 Nasca Octavian Paul
        Author: Nasca Octavian Paul

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <FreeImage.h>
#include "ImageWrapper.h"

struct ImageWrapperInfo{
	ImageWrapperInfo(){
		img=NULL;
	};
	~ImageWrapperInfo(){
		if (img!=NULL) FreeImage_Unload(img);
	};
	FIBITMAP *img;
};

bool ImageWrapper::inited=false;

void ImageWrapper::init(){
	if (inited) return;
	inited=true;
};

void ImageWrapper::deinit(){
	if (!inited) return;
	inited=false;
};


ImageWrapper::ImageWrapper(){
	if (!inited) init();

	ImageWrapperInfo *iwi=new ImageWrapperInfo();
	info=iwi;
	imgdata=NULL;
};


ImageWrapper::~ImageWrapper(){
	ImageWrapperInfo *iwi=(ImageWrapperInfo *)info;
	delete iwi;
	if (imgdata) delete[]imgdata;
};
		
bool ImageWrapper::load(const char *filename){
	ImageWrapperInfo *iwi=(ImageWrapperInfo *)info;
	if (imgdata) delete[]imgdata;
	imgdata=NULL;
	if (iwi->img) FreeImage_Unload(iwi->img);

	FREE_IMAGE_FORMAT fif = FIF_UNKNOWN;
	fif=FreeImage_GetFileType(filename,0);
	if (fif==FIF_UNKNOWN){
		fif = FreeImage_GetFIFFromFilename(filename);
		if (fif==FIF_UNKNOWN) return false;
	};

	iwi->img=FreeImage_Load(fif,filename);
	if (!(iwi->img)) return false;
	if (FreeImage_GetBPP(iwi->img)!=24){
		FIBITMAP *newimg=FreeImage_ConvertTo24Bits(iwi->img); 
		FreeImage_Unload(iwi->img);
		if (newimg) {
			iwi->img=newimg;
		}else{
			return false;
		};
	};

	sizex=FreeImage_GetWidth(iwi->img);
	sizey=FreeImage_GetHeight(iwi->img);
	imgdata=new unsigned char[sizex*sizey*3];

	//convert the image into the internal 8bit RGB format
	for (int j=0;j<sizey;j++){
		const unsigned char *scanline=FreeImage_GetScanLine(iwi->img,sizey-1-j);
		int pos=0;
		int ypos=j*sizex*3;
		for (int i=0;i<sizex;i++){
			imgdata[ypos+pos]=scanline[pos+FI_RGBA_RED];
			imgdata[ypos+pos+1]=scanline[pos+FI_RGBA_GREEN];
			imgdata[ypos+pos+2]=scanline[pos+FI_RGBA_BLUE];
			pos+=3;
		};

	};

	return true;
};

bool ImageWrapper::save(const char *filename){
	ImageWrapperInfo *iwi=(ImageWrapperInfo *)info;
	//convert the image into the External format and save it
	for (int j=0;j<sizey;j++){
		unsigned char *scanline=FreeImage_GetScanLine(iwi->img,sizey-1-j);
		int pos=0;
		int ypos=j*sizex*3;
		for (int i=0;i<sizex;i++){
			scanline[pos]=imgdata[ypos+pos+FI_RGBA_RED];
			scanline[pos+1]=imgdata[ypos+pos+FI_RGBA_GREEN];
			scanline[pos+2]=imgdata[ypos+pos+FI_RGBA_BLUE];
			pos+=3;
		};
	};
	FREE_IMAGE_FORMAT fif = FIF_UNKNOWN;
	fif = FreeImage_GetFIFFromFilename(filename);
	if (fif==FIF_UNKNOWN) fif=FIF_JPEG;
	FreeImage_Save(fif,iwi->img,filename);
	
	return true;
};

